﻿using System;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

namespace JudgeModule
{
    public class NoteJudger
    {
        private Transform deactives;
        private Action EndGame;
        private GameObject judgeText;
        private JudgeTextManager judgetextmanager;
        private MotionGageManager guage;

        public NoteJudger(Dictionary<string, GameObject> obj, Action endgame)
        {
            deactives = obj["deactives"].transform;
            judgeText = obj["judgetext"];
            EndGame = endgame;

            judgetextmanager = judgeText.GetComponent<JudgeTextManager>();
            guage = new MotionGageManager { motionGuage = obj["guage"] };
            guage.ResetGuage();
        }

        public void WrongNoteProc(Note note)
        {
            SetJudge(Judge.MISS, judgeText);
            note.Component.Deactivate(deactives.transform);

            if (!(note is MotionNote))
                EndGame();
        }

        public void EnteredNoteProc(Note note, float timing)
        {
            var judge = Judge.TestJudge(note, timing);

            SetJudge(judge, judgeText);
            judgetextmanager.Register(note);

            Debug.Log("Entered: " + timing + ", Judge: " + judge.Name);

            if (note.IsLong && !judge.IsBreak)
            {
                note.Activated = true;

                if (note.Type == NoteType.LMO)
                    guage.ResetGuage(note.Length, timing);
            }
            else
            {
                note.Component.Deactivate(deactives.transform, !note.IsLong);
                judgetextmanager.UnRegister();
            }
        }

        public void ContinuingNoteProc(Note note, float timing, float interval)
        {
            guage.UpdateGuage(timing);
            if (Judge.IsNoteProgress(note, timing, interval))
            {
                Debug.Log("Continuing: " + timing);
                SetJudge(Judge.Perfect, judgeText);
                judgetextmanager.Register(note);
                ++note.JudgeCount;
            }
        }

        public void CorrectlyStoppedNoteProc(Note note, float timing)
        {
            guage.ResetGuage();
            SetJudge(Judge.TestJudge(note, timing, true), judgeText);
            judgetextmanager.Register(note);
            note.Component.Deactivate(deactives.transform);
            judgetextmanager.UnRegister();
        }

        public void IncorrectlyStoppedNoteProc(Note note)
        {
            guage.ResetGuage();
            SetJudge(Judge.MISS, judgeText);
            judgetextmanager.Register(note);
            note.Component.Deactivate(deactives.transform, false);
            judgetextmanager.UnRegister();
        }

        public static void SetJudge(Judge judge, GameObject judgeText)
        {
            AllSceneManager.Combo = judge.IsBreak ? 0 : AllSceneManager.Combo + 1;
            AllSceneManager.Score += judge.Score;
            AllSceneManager.JudgeCount[judge]++;

            judgeText.SetActive(false);
            judgeText.SetActive(true);
            judgeText.GetComponent<Text>().text = judge.Name;
            judgeText.GetComponent<Text>().color = judge.Color;

            Debug.Log("SetJudge: " + AllSceneManager.TotalJudge + "(" + judge.Name + ")");
        }
    }
}